/*******************************************************************************
  MPLAB Harmony Application Source File

  Company:
    Microchip Technology Inc.

  File Name:
    app.c

  Summary:
    This file contains the source code for the MPLAB Harmony application.

  Description:
    This file contains the source code for the MPLAB Harmony application.  It
    implements the logic of the application's state machine and it may call
    API routines of other MPLAB Harmony modules in the system, such as drivers,
    system services, and middleware.  However, it does not call any of the
    system interfaces (such as the "Initialize" and "Tasks" functions) of any of
    the modules in the system or make any assumptions about when those functions
    are called.  That is the responsibility of the configuration-specific system
    files.
 *******************************************************************************/

// *****************************************************************************
// *****************************************************************************
// Section: Included Files
// *****************************************************************************
// *****************************************************************************

#include "app.h"
#include "cryptoauthlib.h"
#include "atcacert/atcacert_client.h"
#include "atcacert/atcacert_def.h"
#include "atcacert/atcacert_host_hw.h"
#include "atcacert/atcacert_pem.h"
#include "../../project_config.h"

// *****************************************************************************
// *****************************************************************************
// Section: Global Data Definitions
// *****************************************************************************
// *****************************************************************************

extern ATCAIfaceCfg atecc608_0_init_data;
extern ATCAIfaceCfg ta010_0_init_data;     //TA010 I2C Interface
extern ATCAIfaceCfg ta010_1_init_data;     //TA010 SWI Interface

extern const uint8_t g_cert_ca_public_key_1_signer[ATCA_PUB_KEY_SIZE];
extern const atcacert_def_t g_cert_def_1_signer;
extern const atcacert_def_t g_cert_def_3_device;

cal_buffer g_cert_ca_public_key_1_signer_buf = CAL_BUF_INIT(sizeof(g_cert_ca_public_key_1_signer), g_cert_ca_public_key_1_signer);

uint8_t g_signer_cert[1024];
size_t  g_signer_cert_size = 1024;
uint8_t g_signer_public_key[ATCA_PUB_KEY_SIZE];
cal_buffer g_signer_public_key_buf = CAL_BUF_INIT(ATCA_PUB_KEY_SIZE, g_signer_public_key);
uint8_t g_device_cert[1024];
size_t  g_device_cert_size = 1024;
uint8_t g_challenge[RANDOM_NUM_SIZE];
cal_buffer g_challenge_buf = CAL_BUF_INIT(sizeof(g_challenge), g_challenge);
uint8_t g_response[ATCA_SIG_SIZE];
cal_buffer g_response_buf = CAL_BUF_INIT(sizeof(g_response), g_response);

char displaystr[1024];
size_t displaylen;

ATCAIfaceCfg *ta010_cfg = (SELECTED_TA010_SWI) ? &ta010_1_init_data : &ta010_0_init_data;

// *****************************************************************************
/* Application Data

  Summary:
    Holds application data

  Description:
    This structure holds the application's data.

  Remarks:
    This structure should be initialized by the APP_Initialize function.

    Application strings and buffers are be defined outside this structure.
*/

APP_DATA appData;

// *****************************************************************************
// *****************************************************************************
// Section: Application Callback Functions
// *****************************************************************************
// *****************************************************************************

/* TODO:  Add any necessary callback functions.
*/

// *****************************************************************************
// *****************************************************************************
// Section: Application Local Functions
// *****************************************************************************
// *****************************************************************************


ATCA_STATUS rebuild_certificates_from_device(void)
{
    ATCA_STATUS status;

    do
    {
        g_signer_cert_size = sizeof(g_signer_cert);
        if (ATCACERT_E_SUCCESS != (status = atcacert_read_cert(&g_cert_def_1_signer, &g_cert_ca_public_key_1_signer_buf,
                                                            g_signer_cert, &g_signer_cert_size)))
        {
            printf("atcacert_read_cert failed with an error %d\r\n", status);
            break;
        }
        displaylen = sizeof(displaystr);
        atcacert_encode_pem_cert(g_signer_cert, g_signer_cert_size, displaystr, &displaylen);
        printf("DEVICE: Rebuilt Signer Certificate: \r\n%s\r\n", displaystr);

        if (ATCACERT_E_SUCCESS != (status = atcacert_get_subj_public_key(&g_cert_def_1_signer, g_signer_cert,
                                                                      g_signer_cert_size, &g_signer_public_key_buf)))
        {
            printf("atcacert_get_subj_public_key failed with an error %d\r\n", status);
            break;
        }

        g_device_cert_size = sizeof(g_device_cert);
        if (ATCACERT_E_SUCCESS != (status = atcacert_read_cert(&g_cert_def_3_device, &g_signer_public_key_buf,
                                                            g_device_cert, &g_device_cert_size)))
        {
            printf("atcacert_read_cert failed with an error %d\r\n", status);
            break;
        }
        displaylen = sizeof(displaystr);
        atcacert_encode_pem_cert(g_device_cert, g_device_cert_size, displaystr, &displaylen);
        printf("DEVICE: Rebuilt Device Certificate: \r\n%s\r\n", displaystr);

    }
    while(0);

    return status;
}

ATCA_STATUS verify_certificate_chain(void)
{
    ATCA_STATUS status;
    if (ATCACERT_E_SUCCESS == (status = atcacert_verify_cert_hw(atcab_get_device(), &g_cert_def_1_signer, g_signer_cert,
                                                             g_signer_cert_size, &g_cert_ca_public_key_1_signer_buf)))
    {
        printf("HOST: Signer certificate verified against root public key!\r\n");

        if (ATCACERT_E_SUCCESS == (status = atcacert_verify_cert_hw(atcab_get_device(), &g_cert_def_3_device, g_device_cert,
                                                                 g_device_cert_size, &g_signer_public_key_buf)))
        {
            printf("HOST: Device certificate verified against signer public key!\r\n");
        }
        else
        {
            printf("Device certificate is not verified with an error as %d\r\n", status);
        }
    }
    else
    {
        printf("Signer certificate is not verified with an error as %d\r\n", status);
    }

    return status;
}

ATCA_STATUS host_generate_challenge(void)
{
    ATCA_STATUS status;

    if (ATCACERT_E_SUCCESS == (status = atcacert_gen_challenge_hw(atcab_get_device(), &g_challenge_buf)))
    {
        displaylen = sizeof(displaystr);
        atcab_bin2hex(g_challenge, sizeof(g_challenge), displaystr, &displaylen);
        printf("\nHOST: Generated challenge:\r\n%s\r\n\r\n", displaystr);
    }
    else
    {
        printf("Host generate challenge failed with an error %d\r\n", status);
    }

    return status;
}

ATCA_STATUS get_response_from_device_for_host_challenge(void)
{
    ATCA_STATUS status;

    if (ATCACERT_E_SUCCESS == (status = atcacert_get_response(g_cert_def_3_device.private_key_slot,
                                                           &g_challenge_buf, &g_response_buf)))
    {
        displaylen = sizeof(displaystr);
        atcab_bin2hex(g_response, sizeof(g_response), displaystr, &displaylen);
        printf("DEVICE: Calculated response for host challenge:\r\n%s\r\n\r\n", displaystr);
    }
    else
    {
        printf("atcacert_get_response failed with an error %d\r\n", status);
    }

    return status;
}

void verify_device_response(void)
{
    ATCA_STATUS status;
    uint8_t dev_pubkey[ATCA_PUB_KEY_SIZE];
    cal_buffer dev_pubkey_buf = CAL_BUF_INIT(ATCA_PUB_KEY_SIZE, dev_pubkey);

    if (ATCACERT_E_SUCCESS == (status = atcacert_get_subj_public_key(&g_cert_def_3_device, g_device_cert,
                                                                 g_device_cert_size, &dev_pubkey_buf)))
    {
        displaylen = sizeof(displaystr);
        atcab_bin2hex(dev_pubkey, ATCA_PUB_KEY_SIZE, displaystr, &displaylen);
        printf("HOST: Device public key from certificate:\r\n%s\r\n\r\n", displaystr);

        if (ATCACERT_E_SUCCESS == (status = atcacert_verify_response_hw(atcab_get_device(), &dev_pubkey_buf, &g_challenge_buf, &g_response_buf)))
        {
            printf("HOST: Device response to challenge verified!\r\n");
            printf("\nAccessory device authenticated successfully\r\n");
        }
        else
        {
            printf("atcacert_verify_response_hw failed with an error %d\r\n", status);
        }
    }
    else
    {
        printf("atcacert_get_subj_public_key failed with an error %d\r\n", status);
    }

}


// *****************************************************************************
// *****************************************************************************
// Section: Application Initialization and State Machine Functions
// *****************************************************************************
// *****************************************************************************

/*******************************************************************************
  Function:
    void APP_Initialize ( void )

  Remarks:
    See prototype in app.h.
 */

void APP_Initialize ( void )
{
    /* Place the App state machine in its initial state. */
    appData.state = APP_STATE_INIT;



    /* TODO: Initialize your application's state machine and other
     * parameters.
     */
}


/******************************************************************************
  Function:
    void APP_Tasks ( void )

  Remarks:
    See prototype in app.h.
 */

void APP_Tasks ( void )
{
    ATCA_STATUS status;

    /* Check the application's current state. */
    switch ( appData.state )
    {
        /* Application's initial state. */
        case APP_STATE_INIT:
            if(ATCA_SUCCESS != (status = atcab_init(ta010_cfg)))
            {
                printf("atcab_init(ta010_cfg) failed with an error %d\r\n", status);
            }
            appData.state = (ATCA_SUCCESS == status) ? APP_STATE_REBUILD_CERTS : APP_STATE_FINISH;
            break;

        case APP_STATE_REBUILD_CERTS:
            status = rebuild_certificates_from_device();
            appData.state = (ATCACERT_E_SUCCESS == status) ? APP_STATE_VERIFY_CERT_CHAIN : APP_STATE_FINISH;
            break;

        case APP_STATE_VERIFY_CERT_CHAIN:
            if(ATCA_SUCCESS != (status = atcab_init(&atecc608_0_init_data)))
            {
                printf("atcab_init(ecc608_0) failed with an error %d\r\n", status);
            }
            status = verify_certificate_chain();
            appData.state = (ATCACERT_E_SUCCESS == status) ? APP_STATE_HOST_GENERATE_CHALLENGE : APP_STATE_FINISH;
            break;

        case APP_STATE_HOST_GENERATE_CHALLENGE:
            status = host_generate_challenge();
            appData.state = (ATCACERT_E_SUCCESS == status) ? APP_STATE_GET_RESPONSE_FROM_DEVICE : APP_STATE_FINISH;
            break;

        case APP_STATE_GET_RESPONSE_FROM_DEVICE:
            if(ATCA_SUCCESS != (status = atcab_init(ta010_cfg)))
            {
                printf("atcab_init(ta010_cfg) failed with an error %d\r\n", status);
            }
            status = get_response_from_device_for_host_challenge();
            appData.state = (ATCACERT_E_SUCCESS == status) ? APP_STATE_HOST_VERIFY_DEVICE_RESPONSE : APP_STATE_FINISH;
            break;

        case APP_STATE_HOST_VERIFY_DEVICE_RESPONSE:
            if(ATCA_SUCCESS != (status = atcab_init(&atecc608_0_init_data)))
            {
                printf("atcab_init(ecc608_0) failed with an error %d\r\n", status);
            }
            verify_device_response();
            appData.state = APP_STATE_FINISH;
            break;

        case APP_STATE_FINISH:
            break;

        /* TODO: implement your application state machine.*/


        /* The default state should never be executed. */
        default:
        {
            /* TODO: Handle error in application's state machine. */
            break;
        }
    }
}


/*******************************************************************************
 End of File
 */
