/*******************************************************************************
  MPLAB Harmony Application Source File

  Company:
    Microchip Technology Inc.

  File Name:
    app.c

  Summary:
    This file contains the source code for the MPLAB Harmony application.

  Description:
    This file contains the source code for the MPLAB Harmony application.  It
    implements the logic of the application's state machine and it may call
    API routines of other MPLAB Harmony modules in the system, such as drivers,
    system services, and middleware.  However, it does not call any of the
    system interfaces (such as the "Initialize" and "Tasks" functions) of any of
    the modules in the system or make any assumptions about when those functions
    are called.  That is the responsibility of the configuration-specific system
    files.
 *******************************************************************************/

// *****************************************************************************
// *****************************************************************************
// Section: Included Files
// *****************************************************************************
// *****************************************************************************

#include "app.h"
#include "cryptoauthlib.h"
#include "host/atca_host.h"

#include "../../project_config.h"

// *****************************************************************************
// *****************************************************************************
// Section: Global Data Definitions
// *****************************************************************************
// *****************************************************************************

#define HOST_DEVICE_TEMP_KEY_SLOT   (0xFFFF)
#define ACCESSORY_DEVICE_KEY_SLOT   (0x03)
#define SHA_MODE_TARGET_TA010      (0x00)

extern ATCAIfaceCfg atecc608_0_init_data;
extern ATCAIfaceCfg ta010_0_init_data;     //TA010 I2C Interface
extern ATCAIfaceCfg ta010_1_init_data;     //TA010 SWI Interface

uint8_t host_challenge[RANDOM_NUM_SIZE];
uint8_t host_hmac[ATCA_SHA_DIGEST_SIZE];
uint8_t accessory_sernum[ATCA_SERIAL_NUM_SIZE];
uint8_t device_hmac[ATCA_SHA_DIGEST_SIZE];
char displaystr[200];
size_t displaylen;

ATCAIfaceCfg *ta010_cfg = (SELECTED_TA010_SWI) ? &ta010_1_init_data : &ta010_0_init_data;

// *****************************************************************************
/* Application Data

  Summary:
    Holds application data

  Description:
    This structure holds the application's data.

  Remarks:
    This structure should be initialized by the APP_Initialize function.

    Application strings and buffers are be defined outside this structure.
*/

APP_DATA appData;

// *****************************************************************************
// *****************************************************************************
// Section: Application Callback Functions
// *****************************************************************************
// *****************************************************************************

/* TODO:  Add any necessary callback functions.
*/

// *****************************************************************************
// *****************************************************************************
// Section: Application Local Functions
// *****************************************************************************
// *****************************************************************************


ATCA_STATUS generate_host_challenge()
{
    ATCA_STATUS status;

    if (ATCA_SUCCESS == (status = atcab_random(host_challenge)))
    {
        displaylen = sizeof(displaystr);
        atcab_bin2hex(host_challenge, RANDOM_NUM_SIZE, displaystr, &displaylen);
        printf("\nGenerated Host challenge:\r\n%s\r\n", displaystr);
    }
    else
    {
        printf("\nHost challenge generation failed with status {%02X}.\r\n", status);
    }

    return status;
}

ATCA_STATUS get_accessory_device_response(void)
{
    ATCA_STATUS status;
    ATCA_STATUS counter_status;
    uint32_t counter_value=0;

    //! Print accessory counter value pre authentication step...
    //! Displaying this incase of Limited In Use functionality is enabled.
    counter_status = atcab_counter_read(0, &counter_value);
    printf("\nTA010 Monotonic counter read status: %02X , Value:%lu\r\n", counter_status, counter_value);

    if (ATCA_SUCCESS == (status = atcab_sha_hmac(host_challenge, sizeof(host_challenge),
                                                 ACCESSORY_DEVICE_KEY_SLOT, device_hmac, SHA_MODE_TARGET_TA010)))
    {
        displaylen = sizeof(displaystr);
        atcab_bin2hex(device_hmac, ATCA_SHA256_DIGEST_SIZE, displaystr, &displaylen);
        printf("\nHMAC received from Accessory device:\r\n%s\r\n", displaystr);
    }
    else
    {
        printf("\nDevice HMAC calculation failed with status {%02X}.\r\n", status);
    }

    //! Print accessory counter value post authentication step
    counter_status = atcab_counter_read(0, &counter_value);
    printf("\nTA010 Monotonic counter read status: %02X , Value:%lu\r\n", counter_status, counter_value);

    return status;
}

ATCA_STATUS calculate_host_hmac(void)
{
    ATCA_STATUS status;

    if (ATCA_SUCCESS == (status = atcab_kdf(
                                        KDF_MODE_ALG_HKDF | KDF_MODE_SOURCE_SLOT | KDF_MODE_TARGET_TEMPKEY,
                                        0x0005,
                                        KDF_DETAILS_HKDF_MSG_LOC_INPUT | ((uint32_t)sizeof(accessory_sernum) << 24),
                                        accessory_sernum,
                                        NULL,
                                        NULL)))
    {
        if (ATCA_SUCCESS == (status = atcab_sha_hmac(
                                        host_challenge,
                                        sizeof(host_challenge),
                                        HOST_DEVICE_TEMP_KEY_SLOT,
                                        host_hmac,
                                        SHA_MODE_TARGET_OUT_ONLY)))
        {
            displaylen = sizeof(displaystr);
            atcab_bin2hex(host_hmac, ATCA_SHA256_DIGEST_SIZE, displaystr, &displaylen);
            printf("\nHMAC calculated on the Host:\r\n%s\r\n", displaystr);
        }
        else
        {
            printf("\nHost HMAC calculation failed with status {%02X}.\r\n", status);
        }
    }
    else
    {
        printf("\nHost KDF failed with status {%02X}.\r\n", status);
    }

    return status;
}

void compare_accessory_device_and_host_hmacs(void)
{
    if (memcmp(device_hmac, host_hmac, ATCA_SHA256_DIGEST_SIZE) == 0)
    {
        printf("\nAccessory device is authenticated successfully.\r\n");
    }
    else
    {
        printf("\nAccessory device authentication has failed\r\n");
    }
}


// *****************************************************************************
// *****************************************************************************
// Section: Application Initialization and State Machine Functions
// *****************************************************************************
// *****************************************************************************

/*******************************************************************************
  Function:
    void APP_Initialize ( void )

  Remarks:
    See prototype in app.h.
 */

void APP_Initialize ( void )
{
    /* Place the App state machine in its initial state. */
    appData.state = APP_STATE_INIT;



    /* TODO: Initialize your application's state machine and other
     * parameters.
     */
}


/******************************************************************************
  Function:
    void APP_Tasks ( void )

  Remarks:
    See prototype in app.h.
 */

void APP_Tasks ( void )
{
    ATCA_STATUS status;

    /* Check the application's current state. */
    switch ( appData.state )
    {
        /* Application's initial state. */
        case APP_STATE_INIT:
            if (ATCA_SUCCESS != (status = atcab_init(&atecc608_0_init_data)))
            {
                printf("\natcab_init(ecc608_0) failed with status {%02X}.\r\n", status);
            }
            appData.state = (ATCA_SUCCESS == status) ? APP_STATE_GENERATE_CHALLENGE : APP_STATE_FINISH;
            break;

        case APP_STATE_GENERATE_CHALLENGE:
            status = generate_host_challenge();
            appData.state = (ATCA_SUCCESS == status) ? APP_STATE_GET_DEVICE_RESPONSE : APP_STATE_FINISH;
            break;

        case APP_STATE_GET_DEVICE_RESPONSE:
            if (ATCA_SUCCESS == (status = atcab_init(ta010_cfg)))
            {
                status = get_accessory_device_response();
            }
            else
            {
                printf("\natcab_init(ta010_cfg) failed with status {%02X}.\r\n", status);
            }
            appData.state = (ATCA_SUCCESS == status) ? APP_STATE_READ_ACCESSORY_SERIAL_NUMBER : APP_STATE_FINISH;
            break;

        case APP_STATE_READ_ACCESSORY_SERIAL_NUMBER:
            if (ATCA_SUCCESS != (status = atcab_read_serial_number(accessory_sernum)))
            {
                printf("\natcab_read_serial_number failed with status {%02X}.\r\n", status);
            }
            appData.state = (ATCA_SUCCESS == status) ? APP_STATE_CALCULATE_HOST_HMAC : APP_STATE_FINISH;
            break;

        case APP_STATE_CALCULATE_HOST_HMAC:
            if (ATCA_SUCCESS == (status = atcab_init(&atecc608_0_init_data)))
            {
                status = calculate_host_hmac();
            }
            else
            {
                printf("\natcab_init(ecc608_0) failed with status {%02X}.\r\n", status);
            }
            appData.state = (ATCA_SUCCESS == status) ? APP_STATE_COMPARE_HMAC : APP_STATE_FINISH;
            break;

        case APP_STATE_COMPARE_HMAC:
            compare_accessory_device_and_host_hmacs();
            appData.state = APP_STATE_FINISH;
            break;

        case APP_STATE_FINISH:
            break;

        /* The default state should never be executed. */
        default:
        {
            /* TODO: Handle error in application's state machine. */
            break;
        }
    }
}


/*******************************************************************************
 End of File
 */
